<?php

namespace FiloBlu\Hotfix\Plugin;

/**
 * Class StockRegistry
 * @see https://github.com/magento/magento2/issues/15201
 *
 * @package FiloBlu\StockSync\Plugin
 */
class StockRegistry
{
    /**
     * @var \Magento\CatalogInventory\Model\StockRegistryFactory
     */
    protected $stockRegistryFactory;

    /**
     * @var \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory
     */
    protected $configurableFactory;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\StockItemRepository
     */
    protected $stockItemRepository;

    /**
     * @var \Magento\Catalog\Model\ProductRepository
     */
    protected $productRepository;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\CatalogInventory\Model\StockRegistry
     */
    protected $stockRegistry;

    /**
     * StockRegistry constructor.
     * @param \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory $configurableFactory
     * @param \Magento\CatalogInventory\Model\StockRegistryFactory $stockRegistryFactory
     * @param \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository
     * @param \Magento\Catalog\Model\ProductRepository $productRepository
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory $configurableFactory,
        \Magento\CatalogInventory\Model\StockRegistryFactory $stockRegistryFactory,
        \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Psr\Log\LoggerInterface $logger
    )
    {
        $this->configurableFactory = $configurableFactory;
        $this->stockRegistryFactory = $stockRegistryFactory;
        $this->stockItemRepository = $stockItemRepository;
        $this->productRepository = $productRepository;
        $this->logger = $logger;
    }

    /**
     * @param \Magento\CatalogInventory\Model\StockRegistry $subject
     * @param $stockItemId
     * @return mixed
     */
    public function afterUpdateStockItemBySku(\Magento\CatalogInventory\Model\StockRegistry $subject, $stockItemId)
    {
        $this->stockRegistry = $subject;
        return $this->syncProductIsInStockToQuantityAndStockStatus($stockItemId);
    }


    /**
     * @param $stockItemId
     * @return mixed
     */
    protected function syncProductIsInStockToQuantityAndStockStatus($stockItemId)
    {
        try {

            $stockItem = $this->stockItemRepository->get($stockItemId);
            $product = $this->productRepository->getById($stockItem->getProductId());
            $product->addAttributeUpdate('quantity_and_stock_status', $stockItem->getIsInStock(), $product->getStoreId());

        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }

        return $stockItemId;
    }

    /**
     * @param $stockItemId
     * @return mixed
     */
    protected function syncProductStockFromQuantity($stockItemId)
    {
        try {
            $stockRegistry = $this->stockRegistryFactory->create();

            $stockItem = $this->stockItemRepository->get($stockItemId);
            $productId = $stockItem->getProductId();

            $product = $this->productRepository->getById($productId);

            $parents = $this->configurableFactory->create()->getParentIdsByChild($productId);
            $productParent = $this->productRepository->getById($parents[0]);

            $childrenIds = array_values($this->configurableFactory->create()->getChildrenIds($productParent->getId())[0]);
            $sumQuantity = 0;

            foreach ($childrenIds as $childrenId) {
                $sumQuantity += $stockRegistry->getStockItem($childrenId)->getQty();
            }

            $isInStock = $sumQuantity > 0 ? 1 : 0;
            $product->addAttributeUpdate('quantity_and_stock_status', $stockItem->getIsInStock(), $product->getStoreId());
            $productParent->addAttributeUpdate('quantity_and_stock_status', $isInStock, $product->getStoreId());

            $parentStockItem = $this->stockRegistry->getStockItem($productParent->getId());
            $parentStockItem->setIsInStock($isInStock);
            $this->stockItemRepository->save($parentStockItem);
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }

        return $stockItemId;
    }
}